# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from paperpilot_common.protobuf.ai import ai_pb2 as paperpilot__common_dot_protobuf_dot_ai_dot_ai__pb2


class GptServiceStub(object):
    """GPT 服务"""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Ask = channel.unary_stream(
            "/ai.GptService/Ask",
            request_serializer=paperpilot__common_dot_protobuf_dot_ai_dot_ai__pb2.GptRequest.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_ai_dot_ai__pb2.GptResult.FromString,
        )


class GptServiceServicer(object):
    """GPT 服务"""

    def Ask(self, request, context):
        """询问 GPT"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_GptServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "Ask": grpc.unary_stream_rpc_method_handler(
            servicer.Ask,
            request_deserializer=paperpilot__common_dot_protobuf_dot_ai_dot_ai__pb2.GptRequest.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_ai_dot_ai__pb2.GptResult.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler("ai.GptService", rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class GptService(object):
    """GPT 服务"""

    @staticmethod
    def Ask(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_stream(
            request,
            target,
            "/ai.GptService/Ask",
            paperpilot__common_dot_protobuf_dot_ai_dot_ai__pb2.GptRequest.SerializeToString,
            paperpilot__common_dot_protobuf_dot_ai_dot_ai__pb2.GptResult.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import google.protobuf.timestamp_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class ServerStatus(google.protobuf.message.Message):
    """后端状态"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HOST_COUNT_FIELD_NUMBER: builtins.int
    PROJECTS_FIELD_NUMBER: builtins.int
    TIME_FIELD_NUMBER: builtins.int
    host_count: builtins.int
    """后端主机数量"""
    @property
    def projects(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ProjectStatus]:
        """项目状态"""
    @property
    def time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """当前时间"""
    def __init__(
        self,
        *,
        host_count: builtins.int = ...,
        projects: collections.abc.Iterable[global___ProjectStatus] | None = ...,
        time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["time", b"time"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal["host_count", b"host_count", "projects", b"projects", "time", b"time"],
    ) -> None: ...

global___ServerStatus = ServerStatus

@typing_extensions.final
class ProjectStatus(google.protobuf.message.Message):
    """项目状态"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    HEALTHY_COUNT_FIELD_NUMBER: builtins.int
    TOTAL_COUNT_FIELD_NUMBER: builtins.int
    id: builtins.str
    """项目ID(英文)"""
    name: builtins.str
    """项目名(中文)"""
    description: builtins.str
    """项目描述"""
    healthy_count: builtins.int
    """健康实例数量"""
    total_count: builtins.int
    """总实例数量"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        name: builtins.str = ...,
        description: builtins.str = ...,
        healthy_count: builtins.int = ...,
        total_count: builtins.int = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "description",
            b"description",
            "healthy_count",
            b"healthy_count",
            "id",
            b"id",
            "name",
            b"name",
            "total_count",
            b"total_count",
        ],
    ) -> None: ...

global___ProjectStatus = ProjectStatus



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: paperpilot_common/protobuf/monitor/server.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n/paperpilot_common/protobuf/monitor/server.proto\x12\x07monitor\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1bgoogle/protobuf/empty.proto"v\n\x0cServerStatus\x12\x12\n\nhost_count\x18\x01 \x01(\x05\x12(\n\x08projects\x18\x02 \x03(\x0b\x32\x16.monitor.ProjectStatus\x12(\n\x04time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp"j\n\rProjectStatus\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\x15\n\rhealthy_count\x18\x04 \x01(\x05\x12\x13\n\x0btotal_count\x18\x05 \x01(\x05\x32R\n\x14MonitorPublicService\x12:\n\tGetStatus\x12\x16.google.protobuf.Empty\x1a\x15.monitor.ServerStatusb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "paperpilot_common.protobuf.monitor.server_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_SERVERSTATUS"]._serialized_start = 122
    _globals["_SERVERSTATUS"]._serialized_end = 240
    _globals["_PROJECTSTATUS"]._serialized_start = 242
    _globals["_PROJECTSTATUS"]._serialized_end = 348
    _globals["_MONITORPUBLICSERVICE"]._serialized_start = 350
    _globals["_MONITORPUBLICSERVICE"]._serialized_end = 432
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: paperpilot_common/protobuf/project/project.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2

from paperpilot_common.protobuf.user import user_pb2 as paperpilot__common_dot_protobuf_dot_user_dot_user__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n0paperpilot_common/protobuf/project/project.proto\x12\x07project\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a*paperpilot_common/protobuf/user/user.proto"\x17\n\tProjectId\x12\n\n\x02id\x18\x01 \x01(\t"(\n\x11ProjectInviteCode\x12\x13\n\x0binvite_code\x18\x01 \x01(\t"\x84\x01\n\x0bProjectInfo\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\x13\n\x0binvite_code\x18\x04 \x01(\t\x12\x10\n\x08owner_id\x18\x05 \x01(\t\x12\x1f\n\x07members\x18\x06 \x03(\x0b\x32\x0e.user.UserInfo"G\n\x12ListProjectRequest\x12\x0c\n\x04page\x18\x01 \x01(\x05\x12\x11\n\tpage_size\x18\x02 \x01(\x05\x12\x10\n\x08order_by\x18\x03 \x01(\t"_\n\x13ListProjectResponse\x12&\n\x08projects\x18\x01 \x03(\x0b\x32\x14.project.ProjectInfo\x12\r\n\x05total\x18\x02 \x01(\x05\x12\x11\n\tnext_page\x18\x03 \x01(\x05"9\n\x14\x43reateProjectRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t"E\n\x14UpdateProjectRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t"D\n\x1d\x43heckUserJoinedProjectRequest\x12\x12\n\nproject_id\x18\x01 \x01(\t\x12\x0f\n\x07user_id\x18\x02 \x01(\t2n\n\x0eProjectService\x12\\\n\x16\x43heckUserJoinedProject\x12&.project.CheckUserJoinedProjectRequest\x1a\x1a.google.protobuf.BoolValue2\xf0\x03\n\x14ProjectPublicService\x12S\n\x16ListUserJoinedProjects\x12\x1b.project.ListProjectRequest\x1a\x1c.project.ListProjectResponse\x12:\n\x0eGetProjectInfo\x12\x12.project.ProjectId\x1a\x14.project.ProjectInfo\x12\x44\n\rCreateProject\x12\x1d.project.CreateProjectRequest\x1a\x14.project.ProjectInfo\x12H\n\x11UpdateProjectInfo\x12\x1d.project.UpdateProjectRequest\x1a\x14.project.ProjectInfo\x12;\n\rDeleteProject\x12\x12.project.ProjectId\x1a\x16.google.protobuf.Empty\x12?\n\x0bJoinProject\x12\x1a.project.ProjectInviteCode\x1a\x14.project.ProjectInfo\x12\x39\n\x0bQuitProject\x12\x12.project.ProjectId\x1a\x16.google.protobuf.Emptyb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "paperpilot_common.protobuf.project.project_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_PROJECTID"]._serialized_start = 166
    _globals["_PROJECTID"]._serialized_end = 189
    _globals["_PROJECTINVITECODE"]._serialized_start = 191
    _globals["_PROJECTINVITECODE"]._serialized_end = 231
    _globals["_PROJECTINFO"]._serialized_start = 234
    _globals["_PROJECTINFO"]._serialized_end = 366
    _globals["_LISTPROJECTREQUEST"]._serialized_start = 368
    _globals["_LISTPROJECTREQUEST"]._serialized_end = 439
    _globals["_LISTPROJECTRESPONSE"]._serialized_start = 441
    _globals["_LISTPROJECTRESPONSE"]._serialized_end = 536
    _globals["_CREATEPROJECTREQUEST"]._serialized_start = 538
    _globals["_CREATEPROJECTREQUEST"]._serialized_end = 595
    _globals["_UPDATEPROJECTREQUEST"]._serialized_start = 597
    _globals["_UPDATEPROJECTREQUEST"]._serialized_end = 666
    _globals["_CHECKUSERJOINEDPROJECTREQUEST"]._serialized_start = 668
    _globals["_CHECKUSERJOINEDPROJECTREQUEST"]._serialized_end = 736
    _globals["_PROJECTSERVICE"]._serialized_start = 738
    _globals["_PROJECTSERVICE"]._serialized_end = 848
    _globals["_PROJECTPUBLICSERVICE"]._serialized_start = 851
    _globals["_PROJECTPUBLICSERVICE"]._serialized_end = 1347
# @@protoc_insertion_point(module_scope)



"""Base class for server-side interceptors."""

import abc
from asyncio import iscoroutine
from typing import Any, Callable, Tuple

import grpc
from grpc import aio as grpc_aio  # Needed for grpcio pre-1.33.2


class ServerInterceptor(grpc.ServerInterceptor, metaclass=abc.ABCMeta):
    """Base class for server-side interceptors.

    To implement an interceptor, subclass this class and override the intercept method.
    """

    @abc.abstractmethod
    def intercept(
        self,
        method: Callable,
        request_or_iterator: Any,
        context: grpc.ServicerContext,
        method_name: str,
    ) -> Any:  # pragma: no cover
        """Override this method to implement a custom interceptor.

        You should call method(request_or_iterator, context) to invoke the next handler
        (either the RPC method implementation, or the next interceptor in the list).

        Args:
            method: Either the RPC method implementation, or the next interceptor in
                the chain.
            request_or_iterator: The RPC request, as a protobuf message if it is a
                unary request, or an iterator of protobuf messages if it is a streaming
                request.
            context: The ServicerContext pass by gRPC to the service.
            method_name: A string of the form "/protobuf.package.Service/Method"

        Returns:
            This should return the result of method(request, context), which
            is typically the RPC method response, as a protobuf message, or an
            iterator of protobuf messages for streaming responses. The interceptor is
            free to modify this in some way, however.
        """
        return method(request_or_iterator, context)

    # Implementation of grpc.ServerInterceptor, do not override.
    def intercept_service(self, continuation, handler_call_details):
        """Implementation of grpc.ServerInterceptor.

        This is not part of the grpc_interceptor.ServerInterceptor API, but must have
        a public name. Do not override it, unless you know what you're doing.
        """
        next_handler = continuation(handler_call_details)
        # Returns None if the method isn't implemented.
        if next_handler is None:
            return

        handler_factory, next_handler_method = _get_factory_and_method(next_handler)

        def invoke_intercept_method(request_or_iterator, context):
            method_name = handler_call_details.method
            return self.intercept(
                next_handler_method,
                request_or_iterator,
                context,
                method_name,
            )

        return handler_factory(
            invoke_intercept_method,
            request_deserializer=next_handler.request_deserializer,
            response_serializer=next_handler.response_serializer,
        )


class AsyncServerMiddleware(grpc_aio.ServerInterceptor, metaclass=abc.ABCMeta):
    """Base class for asyncio server-side interceptors.

    To implement an interceptor, subclass this class and override the intercept method.
    """

    @abc.abstractmethod
    async def intercept(
        self,
        method: Callable,
        request_or_iterator: Any,
        context: grpc_aio.ServicerContext,
        method_name: str,
    ) -> Any:  # pragma: no cover
        """Override this method to implement a custom interceptor.

        You should await method(request_or_iterator, context) to invoke the next handler
        (either the RPC method implementation, or the next interceptor in the list).

        Args:
            method: Either the RPC method implementation, or the next interceptor in
                the chain.
            request_or_iterator: The RPC request, as a protobuf message if it is a
                unary request, or an iterator of protobuf messages if it is a streaming
                request.
            context: The ServicerContext pass by gRPC to the service.
            method_name: A string of the form "/protobuf.package.Service/Method"

        Returns:
            This should return the result of method(request_or_iterator, context),
            which is typically the RPC method response, as a protobuf message. The
            interceptor is free to modify this in some way, however.
        """
        response_or_iterator = method(request_or_iterator, context)
        if hasattr(response_or_iterator, "__aiter__"):
            return response_or_iterator
        else:
            return await response_or_iterator

    # Implementation of grpc.ServerInterceptor, do not override.
    async def intercept_service(self, continuation, handler_call_details):
        """Implementation of grpc.aio.ServerInterceptor.

        This is not part of the grpc_interceptor.AsyncServerInterceptor API, but must
        have a public name. Do not override it, unless you know what you're doing.
        """
        next_handler = await continuation(handler_call_details)
        if not next_handler:
            return
        handler_factory, next_handler_method = _get_factory_and_method(next_handler)

        if next_handler.response_streaming:

            async def invoke_intercept_method(request, context):
                method_name = handler_call_details.method

                coroutine_or_asyncgen = self.intercept(
                    next_handler_method,
                    request,
                    context,
                    method_name,
                )

                # Async server streaming handlers return async_generator, because they
                # use the async def + yield syntax. However, this is NOT a coroutine
                # and hence is not awaitable. This can be a problem if the interceptor
                # ignores the individual streaming response items and simply returns the
                # result of method(request, context). In that case the interceptor IS a
                # coroutine, and hence should be awaited. In both cases, we need
                # something we can iterate over so that THIS function is an
                # async_generator like the actual RPC method.
                if iscoroutine(coroutine_or_asyncgen):
                    asyncgen_or_none = await coroutine_or_asyncgen
                    # If a handler is using the read/write API, it will return None.
                    if not asyncgen_or_none:
                        return
                    asyncgen = asyncgen_or_none
                else:
                    asyncgen = coroutine_or_asyncgen

                async for r in asyncgen:
                    yield r

        else:

            async def invoke_intercept_method(request, context):
                method_name = handler_call_details.method
                return await self.intercept(
                    next_handler_method,
                    request,
                    context,
                    method_name,
                )

        return handler_factory(
            invoke_intercept_method,
            request_deserializer=next_handler.request_deserializer,
            response_serializer=next_handler.response_serializer,
        )

    async def call(self, method, request, context):
        """Call method in an async way"""
        response = method(request, context)
        if hasattr(response, "__aiter__"):
            return response
        else:
            return await response


def _get_factory_and_method(
    rpc_handler: grpc.RpcMethodHandler,
) -> Tuple[Callable, Callable]:
    if rpc_handler.unary_unary:
        return grpc.unary_unary_rpc_method_handler, rpc_handler.unary_unary
    elif rpc_handler.unary_stream:
        return grpc.unary_stream_rpc_method_handler, rpc_handler.unary_stream
    elif rpc_handler.stream_unary:
        return grpc.stream_unary_rpc_method_handler, rpc_handler.stream_unary
    elif rpc_handler.stream_stream:
        return grpc.stream_stream_rpc_method_handler, rpc_handler.stream_stream
    else:  # pragma: no cover
        raise RuntimeError("RPC handler implementation does not exist")


class MethodName:
    """Represents a gRPC method name.

    gRPC methods are defined by three parts, represented by the three attributes.

    Attributes:
        package: This is defined by the `package foo.bar;` designation in the protocol
            buffer definition, or it could be defined by the protocol buffer directory
            structure, depending on the language
            (see https://developers.google.com/protocol-buffers/docs/proto3#packages).
        service: This is the service name in the protocol buffer definition (e.g.,
            `service SearchService { ... }`.
        method: This is the method name. (e.g., `rpc Search(...) returns (...);`).
    """

    def __init__(self, package: str, service: str, method: str):
        self.package = package
        self.service = service
        self.method = method

    def __repr__(self) -> str:
        """Object-like representation."""
        return f"MethodName(package='{self.package}', service='{self.service}'," f" method='{self.method}')"

    @property
    def fully_qualified_service(self):
        """Return the service name prefixed with the package.

        Example:
            >>> MethodName("foo.bar", "SearchService", "Search").fully_qualified_service
            'foo.bar.SearchService'
        """
        return f"{self.package}.{self.service}" if self.package else self.service


def parse_method_name(method_name: str) -> MethodName:
    """Parse a method name into package, service and endpoint components.

    Arguments:
        method_name: A string of the form "/foo.bar.SearchService/Search", as passed to
            ServerInterceptor.intercept().

    Returns:
        A MethodName object.

    Example:
        >>> parse_method_name("/foo.bar.SearchService/Search")
        MethodName(package='foo.bar', service='SearchService', method='Search')
    """
    _, package_and_service, method = method_name.split("/")
    *maybe_package, service = package_and_service.rsplit(".", maxsplit=1)
    package = maybe_package[0] if maybe_package else ""
    return MethodName(package, service, method)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2

from paperpilot_common.protobuf.im import im_pb2 as paperpilot__common_dot_protobuf_dot_im_dot_im__pb2


class IMServiceStub(object):
    """即时通讯接口"""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.CreateUser = channel.unary_unary(
            "/im.IMService/CreateUser",
            request_serializer=paperpilot__common_dot_protobuf_dot_im_dot_im__pb2.CreateUserRequest.SerializeToString,
            response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
        )
        self.UpdateUser = channel.unary_unary(
            "/im.IMService/UpdateUser",
            request_serializer=paperpilot__common_dot_protobuf_dot_im_dot_im__pb2.UpdateUserRequest.SerializeToString,
            response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
        )
        self.CreateWorkGroup = channel.unary_unary(
            "/im.IMService/CreateWorkGroup",
            request_serializer=paperpilot__common_dot_protobuf_dot_im_dot_im__pb2.CreateWorkGroupRequest.SerializeToString,
            response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
        )
        self.UpdateWorkGroup = channel.unary_unary(
            "/im.IMService/UpdateWorkGroup",
            request_serializer=paperpilot__common_dot_protobuf_dot_im_dot_im__pb2.UpdateWorkGroupRequest.SerializeToString,
            response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
        )
        self.DeleteWorkGroup = channel.unary_unary(
            "/im.IMService/DeleteWorkGroup",
            request_serializer=paperpilot__common_dot_protobuf_dot_im_dot_im__pb2.DeleteWorkGroupRequest.SerializeToString,
            response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
        )
        self.InviteUserToGroup = channel.unary_unary(
            "/im.IMService/InviteUserToGroup",
            request_serializer=paperpilot__common_dot_protobuf_dot_im_dot_im__pb2.InviteUserToGroupRequest.SerializeToString,
            response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
        )
        self.RemoveUserFromGroup = channel.unary_unary(
            "/im.IMService/RemoveUserFromGroup",
            request_serializer=paperpilot__common_dot_protobuf_dot_im_dot_im__pb2.RemoveUserFromGroupRequest.SerializeToString,
            response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
        )


class IMServiceServicer(object):
    """即时通讯接口"""

    def CreateUser(self, request, context):
        """新建用户"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UpdateUser(self, request, context):
        """更新用户"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CreateWorkGroup(self, request, context):
        """创建Work群组"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UpdateWorkGroup(self, request, context):
        """更新Work群组"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeleteWorkGroup(self, request, context):
        """删除Work群组"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def InviteUserToGroup(self, request, context):
        """邀请用户加入群组"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def RemoveUserFromGroup(self, request, context):
        """从群组中移除用户"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_IMServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "CreateUser": grpc.unary_unary_rpc_method_handler(
            servicer.CreateUser,
            request_deserializer=paperpilot__common_dot_protobuf_dot_im_dot_im__pb2.CreateUserRequest.FromString,
            response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
        ),
        "UpdateUser": grpc.unary_unary_rpc_method_handler(
            servicer.UpdateUser,
            request_deserializer=paperpilot__common_dot_protobuf_dot_im_dot_im__pb2.UpdateUserRequest.FromString,
            response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
        ),
        "CreateWorkGroup": grpc.unary_unary_rpc_method_handler(
            servicer.CreateWorkGroup,
            request_deserializer=paperpilot__common_dot_protobuf_dot_im_dot_im__pb2.CreateWorkGroupRequest.FromString,
            response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
        ),
        "UpdateWorkGroup": grpc.unary_unary_rpc_method_handler(
            servicer.UpdateWorkGroup,
            request_deserializer=paperpilot__common_dot_protobuf_dot_im_dot_im__pb2.UpdateWorkGroupRequest.FromString,
            response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
        ),
        "DeleteWorkGroup": grpc.unary_unary_rpc_method_handler(
            servicer.DeleteWorkGroup,
            request_deserializer=paperpilot__common_dot_protobuf_dot_im_dot_im__pb2.DeleteWorkGroupRequest.FromString,
            response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
        ),
        "InviteUserToGroup": grpc.unary_unary_rpc_method_handler(
            servicer.InviteUserToGroup,
            request_deserializer=paperpilot__common_dot_protobuf_dot_im_dot_im__pb2.InviteUserToGroupRequest.FromString,
            response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
        ),
        "RemoveUserFromGroup": grpc.unary_unary_rpc_method_handler(
            servicer.RemoveUserFromGroup,
            request_deserializer=paperpilot__common_dot_protobuf_dot_im_dot_im__pb2.RemoveUserFromGroupRequest.FromString,
            response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler("im.IMService", rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class IMService(object):
    """即时通讯接口"""

    @staticmethod
    def CreateUser(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/im.IMService/CreateUser",
            paperpilot__common_dot_protobuf_dot_im_dot_im__pb2.CreateUserRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def UpdateUser(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/im.IMService/UpdateUser",
            paperpilot__common_dot_protobuf_dot_im_dot_im__pb2.UpdateUserRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def CreateWorkGroup(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/im.IMService/CreateWorkGroup",
            paperpilot__common_dot_protobuf_dot_im_dot_im__pb2.CreateWorkGroupRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def UpdateWorkGroup(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/im.IMService/UpdateWorkGroup",
            paperpilot__common_dot_protobuf_dot_im_dot_im__pb2.UpdateWorkGroupRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DeleteWorkGroup(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/im.IMService/DeleteWorkGroup",
            paperpilot__common_dot_protobuf_dot_im_dot_im__pb2.DeleteWorkGroupRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def InviteUserToGroup(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/im.IMService/InviteUserToGroup",
            paperpilot__common_dot_protobuf_dot_im_dot_im__pb2.InviteUserToGroupRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def RemoveUserFromGroup(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/im.IMService/RemoveUserFromGroup",
            paperpilot__common_dot_protobuf_dot_im_dot_im__pb2.RemoveUserFromGroupRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )


class IMPublicServiceStub(object):
    """即时通讯公开接口"""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.IMAuth = channel.unary_unary(
            "/im.IMPublicService/IMAuth",
            request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_im_dot_im__pb2.IMAuthResponse.FromString,
        )


class IMPublicServiceServicer(object):
    """即时通讯公开接口"""

    def IMAuth(self, request, context):
        """IM认证"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_IMPublicServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "IMAuth": grpc.unary_unary_rpc_method_handler(
            servicer.IMAuth,
            request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_im_dot_im__pb2.IMAuthResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler("im.IMPublicService", rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class IMPublicService(object):
    """即时通讯公开接口"""

    @staticmethod
    def IMAuth(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/im.IMPublicService/IMAuth",
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            paperpilot__common_dot_protobuf_dot_im_dot_im__pb2.IMAuthResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



from django.db import models as django_models
from django.db.models.signals import class_prepared


def override_field(sender, **kwargs):
    if sender.__name__ == "LogEntry":
        user_id = django_models.IntegerField(default=0, verbose_name="user")
        sender._meta.local_fields = [f for f in sender._meta.fields if f.name not in ["user"]]
        user_id.contribute_to_class(sender, "user_id")


class_prepared.connect(override_field)



from paperpilot_common.utils.log import get_logger


class Updater:
    fields = []
    logger_name = "updater"

    def __init__(self):
        self.logger = get_logger(self.logger_name)

    async def _update_field(self, field: str, obj, vo) -> None:
        # 寻找diff方法（返回True则进行更新）
        diff_method = getattr(self, f"diff_{field}", None)
        if diff_method:  # 存在自定义diff方法
            if await diff_method(obj, vo) is False:  # diff失败，无需更新
                return
        else:
            if getattr(obj, field) == getattr(vo, field):  # 无需更新
                return

        # 寻找validate方法（校验字段）
        validate_method = getattr(self, f"validate_{field}", None)
        if validate_method:  # 存在自定义validate方法
            await validate_method(obj, vo)

        # 寻找update方法（更新字段）
        update_method = getattr(self, f"update_{field}", None)
        if update_method:  # 存在自定义update方法
            await update_method(obj, vo)
        else:
            setattr(obj, field, getattr(vo, field))

        self.logger.debug(f"update user field success, user: {obj}, field: {field}")

    async def update(self, obj, vo, save=False) -> None:
        """
        更新obj信息

        针对每个field：先diff，再validate，最后update

        :param obj: 对象
        :param vo: 更新vo
        :param save: 是否保存(默认不保存)
        """
        for field in self.fields:  # 遍历待更新字段
            await self._update_field(field, obj, vo)

        if save:
            await obj.asave()
            await obj.arefresh_from_db()

        self.logger.debug(f"update user success, user: {obj}")



from paperpilot_common.response import ResponseType

response_type_enum: dict[str, "ResponseType"] | None = None


def get_response_type(code: str) -> "ResponseType":
    """
    根据状态码code取枚举

    :param code: 状态码code
    :return: 枚举
    """
    global response_type_enum

    if response_type_enum is None:
        response_type_enum = {e.value[0]: e for e in ResponseType if isinstance(e.value, tuple)}
    return response_type_enum[code]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class TranslationRequest(google.protobuf.message.Message):
    """翻译请求"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CONTENT_FIELD_NUMBER: builtins.int
    SOURCE_LANGUAGE_FIELD_NUMBER: builtins.int
    TARGET_LANGUAGE_FIELD_NUMBER: builtins.int
    content: builtins.str
    """翻译内容"""
    source_language: builtins.str
    """源语言"""
    target_language: builtins.str
    """目标语言"""
    def __init__(
        self,
        *,
        content: builtins.str = ...,
        source_language: builtins.str | None = ...,
        target_language: builtins.str | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing_extensions.Literal[
            "_source_language",
            b"_source_language",
            "_target_language",
            b"_target_language",
            "source_language",
            b"source_language",
            "target_language",
            b"target_language",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "_source_language",
            b"_source_language",
            "_target_language",
            b"_target_language",
            "content",
            b"content",
            "source_language",
            b"source_language",
            "target_language",
            b"target_language",
        ],
    ) -> None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing_extensions.Literal["_source_language", b"_source_language"]
    ) -> typing_extensions.Literal["source_language"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing_extensions.Literal["_target_language", b"_target_language"]
    ) -> typing_extensions.Literal["target_language"] | None: ...

global___TranslationRequest = TranslationRequest

@typing_extensions.final
class TranslationResponse(google.protobuf.message.Message):
    """翻译响应"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESULT_FIELD_NUMBER: builtins.int
    result: builtins.str
    """翻译结果"""
    def __init__(
        self,
        *,
        result: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["result", b"result"]) -> None: ...

global___TranslationResponse = TranslationResponse



import datetime
from dataclasses import MISSING
from typing import Any, Optional

from google.protobuf.timestamp_pb2 import Timestamp


def timestamp_to_datetime(t: Timestamp, default: Any = MISSING) -> datetime.datetime:
    """replace proto.timestamp to python datetime.datetime"""
    if t.seconds == 0 and t.nanos == 0 and default != MISSING:
        return default
    return t.ToDatetime()


def timestamp_to_date(t: Timestamp, default: Any = MISSING) -> datetime.date:
    """replace proto.timestamp to python datetime.date"""
    if t.seconds == 0 and t.nanos == 0 and default != MISSING:
        return default
    return t.ToDatetime().date()


def datetime_to_timestamp(d: Optional[datetime.datetime]) -> Timestamp:
    """replace python datetime.datetime to proto.timestamp"""
    t: Timestamp = Timestamp()
    if d:
        t.FromDatetime(d)
    return t


def date_to_timestamp(d: Optional[datetime.date]) -> Timestamp:
    """replace python datetime.date to proto.timestamp"""
    t: Timestamp = Timestamp()
    if d:
        t.FromDatetime(datetime.datetime.combine(d, datetime.datetime.min.time()))
    return t



from typing import Any, Callable, Iterator, List, Optional, Tuple, Union

import grpc

GRPC_RESPONSE = Union[grpc.Call, grpc.Future]


class ClientCallDetailsType(grpc.ClientCallDetails):
    method: str
    timeout: Optional[float]
    metadata: Optional[List[Tuple[str, Union[str, bytes]]]]
    credentials: Optional[grpc.CallCredentials]
    wait_for_ready: Optional[bool]
    compression: Optional[grpc.Compression]


class BaseInterceptor(
    grpc.UnaryUnaryClientInterceptor,
    grpc.UnaryStreamClientInterceptor,
    grpc.StreamUnaryClientInterceptor,
    grpc.StreamStreamClientInterceptor,
):
    """Base class for client-side interceptors.

    To implement an interceptor, subclass this class and override the intercept method.
    """

    def intercept(
        self,
        method: Callable,
        request_or_iterator: Any,
        call_details: ClientCallDetailsType,
    ) -> GRPC_RESPONSE:
        """Override this method to implement a custom interceptor.

        This method is called for all unary and streaming RPCs. The interceptor
        implementation should call `method` using a `grpc.ClientCallDetails` and the
        `request_or_iterator` object as parameters. The `request_or_iterator`
        parameter may be type checked to determine if this is a singluar request
        for unary RPCs or an iterator for client-streaming or client-server streaming
        RPCs.

        Args:
            method: A function that proceeds with the invocation by executing the next
                interceptor in the chain or invoking the actual RPC on the underlying
                channel.
            request_or_iterator: RPC request message or iterator of request messages
                for streaming requests.
            call_details: Describes an RPC to be invoked.

        Returns:
            The type of the return should match the type of the return value received
            by calling `method`. This is an object that is both a
            `Call <https://grpc.github.io/grpc/python/grpc.html#grpc.Call>`_ for the
            RPC and a
            `Future <https://grpc.github.io/grpc/python/grpc.html#grpc.Future>`_.

            The actual result from the RPC can be got by calling `.result()` on the
            value returned from `method`.
        """
        return method(call_details, request_or_iterator)  # pragma: no cover

    def intercept_unary_unary(
        self,
        continuation: Callable,
        call_details: ClientCallDetailsType,
        request: Any,
    ) -> GRPC_RESPONSE:
        """Implementation of grpc.UnaryUnaryClientInterceptor.

        This is not part of the grpc_interceptor.ClientInterceptor API, but must have
        a public name. Do not override it, unless you know what you're doing.
        """
        return self.intercept(continuation, request, call_details)

    def intercept_unary_stream(
        self,
        continuation: Callable,
        call_details: ClientCallDetailsType,
        request: Any,
    ) -> GRPC_RESPONSE:
        """Implementation of grpc.UnaryStreamClientInterceptor.

        This is not part of the grpc_interceptor.ClientInterceptor API, but must have
        a public name. Do not override it, unless you know what you're doing.
        """
        return self.intercept(continuation, request, call_details)

    def intercept_stream_unary(
        self,
        continuation: Callable,
        call_details: ClientCallDetailsType,
        request_iterator: Iterator[Any],
    ) -> GRPC_RESPONSE:
        """Implementation of grpc.StreamUnaryClientInterceptor.

        This is not part of the grpc_interceptor.ClientInterceptor API, but must have
        a public name. Do not override it, unless you know what you're doing.
        """
        return self.intercept(continuation, request_iterator, call_details)

    def intercept_stream_stream(
        self,
        continuation: Callable,
        call_details: ClientCallDetailsType,
        request_iterator: Iterator[Any],
    ) -> GRPC_RESPONSE:
        """Implementation of grpc.StreamStreamClientInterceptor.

        This is not part of the grpc_interceptor.ClientInterceptor API, but must have
        a public name. Do not override it, unless you know what you're doing.
        """
        return self.intercept(continuation, request_iterator, call_details)



import base64
import datetime
import hashlib
import hmac
import json

from django.conf import settings

from ..middleware.server.context import get_trace_id
from .configs import oss_settings
from .utils import Condition, OssDirectToken, get_iso_8601, handle_condition, parse_size

default_callback_body = (
    "object=${object}&"
    "size=${size}&"
    "mime_type=${mimeType}&"
    "height=${imageInfo.height}&"
    "width=${imageInfo.width}&"
    "etag=${etag}&"
    "client_ip=${clientIp}"
)


def generate_direct_upload_token(
    callback_url: str,
    callback_body: str = default_callback_body,
    min_size: int | str = 0,
    max_size: int | str = oss_settings.MAX_SIZE_MB,
    key: str | list[str] | Condition = None,
    cache_control: str | list[str] | Condition = None,
    content_type: str | list[str] | Condition = None,
    content_disposition: str | list[str] | Condition = None,
    content_encoding: str | list[str] | Condition = None,
    expires: str | list[str] | Condition = None,
    success_action_redirect: str | list[str] | Condition = None,
    success_action_status: str | list[str] | Condition = None,
    x_oss_meta: dict[str, str | list[str] | Condition] = None,
) -> OssDirectToken:
    """
    获取直传 Token

    :param callback_url: 回调地址（不含host）
    :param callback_body: 回调内容，形如key=${object}&etag=${etag}&my_var=${x:my_var}
    :param min_size: 最小文件大小
    :param max_size: 最大文件大小
    :param key: 文件存储路径（含文件名，不包括MEDIA_URL）
    :param cache_control: 缓存控制（HTTP请求Header）
    :param content_type: 类型（HTTP请求Header）
    :param content_disposition: 附件（HTTP请求Header）
    :param content_encoding: 编码（HTTP请求Header）
    :param expires: 缓存过期时间（HTTP请求Header）
    :param success_action_redirect: 上传成功后跳转地址
    :param success_action_status: 未指定success_action_redirect时，上传成功后的返回状态码
    :param x_oss_meta: 自定义元数据，限定表单域的行为
    :return: 直传 Token
    """
    # 统一callback_url格式
    if callback_url.startswith("/"):
        callback_url = callback_url[1:]
    if oss_settings.CALLBACK_BASE_URL.endswith("/"):
        callback_base_url = oss_settings.CALLBACK_BASE_URL[:-1]
    else:
        callback_base_url = oss_settings.CALLBACK_BASE_URL
    callback_complete_url = f"{callback_base_url}/{callback_url}"

    # 获取Policy，完整参数详见
    # https://help.aliyun.com/zh/oss/developer-reference/postobject?spm=a2c4g.11186623.0.i26#section-d5z-1ww-wdb

    # 处理限制
    conditions = [
        {"bucket": oss_settings.BUCKET_NAME},
        ["content-length-range", parse_size(min_size), parse_size(max_size)],  # 限制上传文件大小
    ]

    upload_key = f"{settings.MEDIA_URL}{key}"
    if upload_key.startswith("/"):
        upload_key = upload_key[1:]

    handle_condition(conditions, "key", upload_key)
    handle_condition(conditions, "cache-control", cache_control)
    handle_condition(conditions, "content-type", content_type)
    handle_condition(conditions, "content-disposition", content_disposition)
    handle_condition(conditions, "content-encoding", content_encoding)
    handle_condition(conditions, "expires", expires)
    handle_condition(conditions, "success_action_redirect", success_action_redirect)
    handle_condition(conditions, "success_action_status", success_action_status)

    if x_oss_meta:
        for k, v in x_oss_meta.items():
            handle_condition(conditions, f"x-oss-meta-{k}", v)

    expire_time = datetime.datetime.now() + datetime.timedelta(seconds=oss_settings.TOKEN_EXPIRE_SECOND)
    expire = get_iso_8601(expire_time.timestamp())
    policy = {
        "expiration": expire,  # 过期时间
        "conditions": conditions,
    }
    policy = json.dumps(policy).strip()
    policy_encode = base64.b64encode(policy.encode())

    # 签名
    h = hmac.new(
        oss_settings.ACCESS_KEY_SECRET.encode(),
        policy_encode,
        hashlib.sha1,
    )
    sign = base64.encodebytes(h.digest()).strip()

    callback_body = f"{callback_body}&trace_id={get_trace_id().hex}"

    # 回调参数，详见
    # https://help.aliyun.com/zh/oss/developer-reference/callback?spm=a2c4g.11186623.0.i74#a8a8e930e31fv
    callback_dict = {
        "callbackUrl": callback_complete_url,
        "callbackBody": callback_body,
        "callbackBodyType": "application/x-www-form-urlencoded",
    }

    callback_param = json.dumps(callback_dict).strip()
    base64_callback_body = base64.b64encode(callback_param.encode())

    host = (
        f'{oss_settings.ENDPOINT.split("://")[0]}://{oss_settings.BUCKET_NAME}.'
        f'{oss_settings.ENDPOINT.split("://")[1]}'
    )

    return OssDirectToken(
        access_key_id=oss_settings.ACCESS_KEY_ID,
        host=host,
        policy=policy_encode.decode(),
        signature=sign.decode(),
        callback=base64_callback_body.decode(),
    )



import inspect
import logging
from typing import Any, Callable, Dict, List, Optional, Type

from google.rpc import status_pb2
from grpc_status import rpc_status

from paperpilot_common.helper.context import context_proxy
from paperpilot_common.protos.common.exce_pb2 import Exec

from .base import GRPC_RESPONSE, BaseInterceptor, ClientCallDetailsType

logger: logging.Logger = logging.getLogger()


class CustomerTopInterceptor(BaseInterceptor):
    def __init__(self, exc_list: Optional[List[Type[Exception]]] = None):
        self.exc_dict: Dict[str, Type[Exception]] = {}
        for key, exc in globals()["__builtins__"].items():
            if inspect.isclass(exc) and issubclass(exc, Exception):
                self.exc_dict[key] = exc

        if exc_list:
            for exc in exc_list:
                if issubclass(exc, Exception):
                    self.exc_dict[exc.__name__] = exc

    def intercept(
        self,
        method: Callable,
        request_or_iterator: Any,
        call_details: ClientCallDetailsType,
    ) -> GRPC_RESPONSE:
        if call_details.metadata is not None:
            call_details.metadata.append(("request_id", context_proxy.request_id))
        response: GRPC_RESPONSE = method(call_details, request_or_iterator)
        status: Optional[status_pb2.Status] = rpc_status.from_call(response)
        if status:
            for detail in status.details:
                if detail.Is(Exec.DESCRIPTOR):
                    exec_instance: Exec = Exec()
                    detail.Unpack(exec_instance)
                    exec_class: Type[Exception] = self.exc_dict.get(exec_instance.name) or RuntimeError
                    raise exec_class(exec_instance.msg)
                else:
                    raise RuntimeError("Unexpected failure: %s" % detail)
        return response



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: paperpilot_common/protobuf/common/exce.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n,paperpilot_common/protobuf/common/exce.proto\x12\x04\x65xec\x1a\x1fgoogle/protobuf/timestamp.proto"c\n\x0c\x41piException\x12\x0c\n\x04\x63ode\x18\x01 \x01(\t\x12\x0e\n\x06\x64\x65tail\x18\x02 \x01(\t\x12\x0f\n\x07message\x18\x03 \x01(\t\x12$\n\x04\x64\x61ta\x18\x04 \x01(\x0b\x32\x16.exec.ApiExceptionData"\x7f\n\x10\x41piExceptionData\x12\x0b\n\x03\x65id\x18\x01 \x01(\t\x12\x11\n\tsentry_id\x18\x02 \x01(\t\x12(\n\x04time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12!\n\x04info\x18\x04 \x01(\x0b\x32\x13.exec.ExceptionInfo"h\n\rExceptionInfo\x12\x0c\n\x04type\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\x12\x11\n\ttraceback\x18\x03 \x03(\t\x12\x12\n\ninner_type\x18\x04 \x01(\t\x12\x13\n\x0binner_value\x18\x05 \x01(\tb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "paperpilot_common.protobuf.common.exce_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_APIEXCEPTION"]._serialized_start = 87
    _globals["_APIEXCEPTION"]._serialized_end = 186
    _globals["_APIEXCEPTIONDATA"]._serialized_start = 188
    _globals["_APIEXCEPTIONDATA"]._serialized_end = 315
    _globals["_EXCEPTIONINFO"]._serialized_start = 317
    _globals["_EXCEPTIONINFO"]._serialized_end = 421
# @@protoc_insertion_point(module_scope)



"""
Django Admin Logs - Model Admin.
"""
from django.contrib import admin
from django.contrib.admin.models import LogEntry
from django.utils.html import format_html


class LogEntryAdmin(admin.ModelAdmin):
    """Log Entry admin interface."""

    date_hierarchy = "action_time"
    fields = (
        "action_time",
        "content_type",
        "object_id",
        "object_repr",
        "action_flag",
        "change_message",
    )
    list_display = (
        "action_time",
        "action_message",
        "content_type",
        "object_link",
    )
    list_filter = (
        "action_flag",
        "content_type",
    )
    search_fields = (
        "object_repr",
        "change_message",
    )

    def object_link(self, obj):
        """Returns the admin link to the log entry object if it exists."""
        admin_url = None if obj.is_deletion() else obj.get_admin_url()
        if admin_url:
            return format_html('<a href="{}">{}</a>', admin_url, obj.object_repr)
        else:
            return obj.object_repr

    object_link.short_description = "object"

    def action_message(self, obj):
        """
        Returns the action message.
        Note: this handles deletions which don't return a change message.
        """
        change_message = obj.get_change_message()
        # If there is no change message then use the action flag label
        if not change_message:
            change_message = "{}.".format(obj.get_action_flag_display())
        return change_message

    action_message.short_description = "action"

    def get_queryset(self, request):
        return super().get_queryset(request).prefetch_related("content_type")

    def has_add_permission(self, request):
        """Log entries cannot be added manually."""
        return False

    def has_change_permission(self, request, obj=None):
        """Log entries cannot be changed."""
        return False

    def has_delete_permission(self, request, obj=None):
        """Log entries can only be deleted when the setting is enabled."""
        return super().has_delete_permission(request, obj)

    # Prevent changes to log entries creating their own log entries!
    def log_addition(self, request, object, message):
        pass

    def log_change(self, request, object, message):
        pass

    def log_deletion(self, request, object, object_repr):
        pass


admin.site.register(LogEntry, LogEntryAdmin)



import time
import uuid
from typing import Any, Callable
from uuid import UUID

import grpc
from grpc_status import rpc_status

from paperpilot_common.exceptions.handler import ApiExceptionHandler
from paperpilot_common.middleware.server.base import AsyncServerMiddleware, parse_method_name
from paperpilot_common.middleware.server.context import get_trace_id, trace_id_context
from paperpilot_common.utils.log import get_logger


class TraceMiddleware(AsyncServerMiddleware):
    logger = get_logger("server.interceptor.trace")
    handler = ApiExceptionHandler()

    async def intercept(
        self,
        method: Callable,
        request_or_iterator: Any,
        context: grpc.ServicerContext,
        method_name: str,
    ) -> Any:
        start_time: float = time.time()

        # 获取request trace id
        trace_id = dict(context.invocation_metadata()).get("x-trace-id", None)
        if trace_id:
            trace_id_context.set(UUID(trace_id))
        else:
            trace_id_context.set(uuid.uuid4())

        try:
            # run grpc handler
            response = await self.call(method, request_or_iterator, context)
            # add trace_id to metadata
            context.set_trailing_metadata((("x-trace-id", trace_id_context.get().hex),))
            return response
        except Exception as e:
            status = self.handler.grpc_handle(e)
            await context.abort_with_status(rpc_status.to_status(status))
        finally:
            method_info = parse_method_name(method_name)
            if method_info.method not in ["ServerReflectionInfo", "Check", "Watch"]:
                self.logger.debug(
                    f"Got Request {get_trace_id()}. method:{method_info.method}, code:{context.code()}, detail:{context.details()}, duration:{time.time() - start_time}"
                )



from django.db import connections


def close_old_connections():
    for conn in connections.all():
        conn.close_if_unusable_or_obsolete()


def db_sync(func):
    """数据库连接关闭装饰器"""

    def func_wrapper(*args):
        close_old_connections()
        result = func(*args)
        close_old_connections()
        return result

    return func_wrapper


def db(func):
    """数据库连接关闭装饰器"""

    async def func_wrapper(*args):
        close_old_connections()
        result = await func(*args)
        close_old_connections()
        return result

    return func_wrapper



import uuid
from contextvars import ContextVar
from uuid import UUID

trace_id_context: ContextVar[UUID] = ContextVar("trace_id", default=UUID(int=0))


def get_trace_id() -> UUID:
    """
    获取trace_id
    """
    trace_id = trace_id_context.get()
    if trace_id is None:
        trace_id = uuid.uuid4()
    return trace_id



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import collections.abc
import grpc
import grpc.aio
import paperpilot_common.protobuf.ai.ai_pb2
import typing

_T = typing.TypeVar("_T")

class _MaybeAsyncIterator(collections.abc.AsyncIterator[_T], collections.abc.Iterator[_T], metaclass=abc.ABCMeta): ...

class _ServicerContext(grpc.ServicerContext, grpc.aio.ServicerContext):  # type: ignore
    ...

class GptServiceStub:
    """GPT 服务"""

    def __init__(self, channel: typing.Union[grpc.Channel, grpc.aio.Channel]) -> None: ...
    Ask: grpc.UnaryStreamMultiCallable[
        paperpilot_common.protobuf.ai.ai_pb2.GptRequest,
        paperpilot_common.protobuf.ai.ai_pb2.GptResult,
    ]
    """询问 GPT"""

class GptServiceAsyncStub:
    """GPT 服务"""

    Ask: grpc.aio.UnaryStreamMultiCallable[
        paperpilot_common.protobuf.ai.ai_pb2.GptRequest,
        paperpilot_common.protobuf.ai.ai_pb2.GptResult,
    ]
    """询问 GPT"""

class GptServiceServicer(metaclass=abc.ABCMeta):
    """GPT 服务"""

    @abc.abstractmethod
    def Ask(
        self,
        request: paperpilot_common.protobuf.ai.ai_pb2.GptRequest,
        context: _ServicerContext,
    ) -> typing.Union[
        collections.abc.Iterator[paperpilot_common.protobuf.ai.ai_pb2.GptResult],
        collections.abc.AsyncIterator[paperpilot_common.protobuf.ai.ai_pb2.GptResult],
    ]:
        """询问 GPT"""

def add_GptServiceServicer_to_server(
    servicer: GptServiceServicer, server: typing.Union[grpc.Server, grpc.aio.Server]
) -> None: ...



import logging
from functools import wraps
from typing import Any, Callable, Type

from google.protobuf.message import Message  # type: ignore

logger: logging.Logger = logging.getLogger()


def ignore_grpc_exc(*, message: Type[Message]) -> Callable[[Callable[..., Message]], Callable[..., Message]]:
    def wrapper(func: Callable[..., Message]) -> Callable[..., Message]:
        @wraps(func)
        def _ignore_exc(*args: Any, **kwargs: Any) -> Message:
            try:
                return func(*args, **kwargs)
            except Exception as e:
                logger.exception(f"call {func._method} error: {e}")  # type: ignore
                return message()

        return _ignore_exc

    return wrapper


def grpc_client_func_wrapper(*args: Any, **kwargs: Any) -> Callable:
    def wrapper(func: Callable) -> Callable:
        @wraps(func)
        def _wrapper(*_args: Any, **_kwargs: Any) -> Any:
            if "metadata" not in _kwargs:
                _kwargs["metadata"] = []
            return func(*_args, **_kwargs)

        return _wrapper

    return wrapper


def auto_load_wrapper_by_stub(stub: Any, wrapper: Callable) -> None:
    for stub_method in dir(stub):
        if stub_method.startswith("__"):
            continue
        grpc_handler: Callable = getattr(stub, stub_method)
        setattr(stub, stub_method, wrapper()(grpc_handler))



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2

from paperpilot_common.protobuf.project import (
    project_pb2 as paperpilot__common_dot_protobuf_dot_project_dot_project__pb2,
)


class ProjectServiceStub(object):
    """项目接口"""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.CheckUserJoinedProject = channel.unary_unary(
            "/project.ProjectService/CheckUserJoinedProject",
            request_serializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.CheckUserJoinedProjectRequest.SerializeToString,
            response_deserializer=google_dot_protobuf_dot_wrappers__pb2.BoolValue.FromString,
        )


class ProjectServiceServicer(object):
    """项目接口"""

    def CheckUserJoinedProject(self, request, context):
        """检查用户是否加入项目"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_ProjectServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "CheckUserJoinedProject": grpc.unary_unary_rpc_method_handler(
            servicer.CheckUserJoinedProject,
            request_deserializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.CheckUserJoinedProjectRequest.FromString,
            response_serializer=google_dot_protobuf_dot_wrappers__pb2.BoolValue.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler("project.ProjectService", rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class ProjectService(object):
    """项目接口"""

    @staticmethod
    def CheckUserJoinedProject(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/project.ProjectService/CheckUserJoinedProject",
            paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.CheckUserJoinedProjectRequest.SerializeToString,
            google_dot_protobuf_dot_wrappers__pb2.BoolValue.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )


class ProjectPublicServiceStub(object):
    """项目公开接口"""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ListUserJoinedProjects = channel.unary_unary(
            "/project.ProjectPublicService/ListUserJoinedProjects",
            request_serializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ListProjectRequest.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ListProjectResponse.FromString,
        )
        self.GetProjectInfo = channel.unary_unary(
            "/project.ProjectPublicService/GetProjectInfo",
            request_serializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ProjectId.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ProjectInfo.FromString,
        )
        self.CreateProject = channel.unary_unary(
            "/project.ProjectPublicService/CreateProject",
            request_serializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.CreateProjectRequest.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ProjectInfo.FromString,
        )
        self.UpdateProjectInfo = channel.unary_unary(
            "/project.ProjectPublicService/UpdateProjectInfo",
            request_serializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.UpdateProjectRequest.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ProjectInfo.FromString,
        )
        self.DeleteProject = channel.unary_unary(
            "/project.ProjectPublicService/DeleteProject",
            request_serializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ProjectId.SerializeToString,
            response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
        )
        self.JoinProject = channel.unary_unary(
            "/project.ProjectPublicService/JoinProject",
            request_serializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ProjectInviteCode.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ProjectInfo.FromString,
        )
        self.QuitProject = channel.unary_unary(
            "/project.ProjectPublicService/QuitProject",
            request_serializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ProjectId.SerializeToString,
            response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
        )


class ProjectPublicServiceServicer(object):
    """项目公开接口"""

    def ListUserJoinedProjects(self, request, context):
        """获取用户参与的项目列表"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetProjectInfo(self, request, context):
        """获取项目信息"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CreateProject(self, request, context):
        """创建项目"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UpdateProjectInfo(self, request, context):
        """修改项目信息"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeleteProject(self, request, context):
        """删除项目"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def JoinProject(self, request, context):
        """加入项目"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def QuitProject(self, request, context):
        """退出项目"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_ProjectPublicServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "ListUserJoinedProjects": grpc.unary_unary_rpc_method_handler(
            servicer.ListUserJoinedProjects,
            request_deserializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ListProjectRequest.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ListProjectResponse.SerializeToString,
        ),
        "GetProjectInfo": grpc.unary_unary_rpc_method_handler(
            servicer.GetProjectInfo,
            request_deserializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ProjectId.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ProjectInfo.SerializeToString,
        ),
        "CreateProject": grpc.unary_unary_rpc_method_handler(
            servicer.CreateProject,
            request_deserializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.CreateProjectRequest.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ProjectInfo.SerializeToString,
        ),
        "UpdateProjectInfo": grpc.unary_unary_rpc_method_handler(
            servicer.UpdateProjectInfo,
            request_deserializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.UpdateProjectRequest.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ProjectInfo.SerializeToString,
        ),
        "DeleteProject": grpc.unary_unary_rpc_method_handler(
            servicer.DeleteProject,
            request_deserializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ProjectId.FromString,
            response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
        ),
        "JoinProject": grpc.unary_unary_rpc_method_handler(
            servicer.JoinProject,
            request_deserializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ProjectInviteCode.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ProjectInfo.SerializeToString,
        ),
        "QuitProject": grpc.unary_unary_rpc_method_handler(
            servicer.QuitProject,
            request_deserializer=paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ProjectId.FromString,
            response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler("project.ProjectPublicService", rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class ProjectPublicService(object):
    """项目公开接口"""

    @staticmethod
    def ListUserJoinedProjects(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/project.ProjectPublicService/ListUserJoinedProjects",
            paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ListProjectRequest.SerializeToString,
            paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ListProjectResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetProjectInfo(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/project.ProjectPublicService/GetProjectInfo",
            paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ProjectId.SerializeToString,
            paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ProjectInfo.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def CreateProject(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/project.ProjectPublicService/CreateProject",
            paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.CreateProjectRequest.SerializeToString,
            paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ProjectInfo.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def UpdateProjectInfo(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/project.ProjectPublicService/UpdateProjectInfo",
            paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.UpdateProjectRequest.SerializeToString,
            paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ProjectInfo.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DeleteProject(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/project.ProjectPublicService/DeleteProject",
            paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ProjectId.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def JoinProject(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/project.ProjectPublicService/JoinProject",
            paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ProjectInviteCode.SerializeToString,
            paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ProjectInfo.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def QuitProject(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/project.ProjectPublicService/QuitProject",
            paperpilot__common_dot_protobuf_dot_project_dot_project__pb2.ProjectId.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



from threading import RLock


class Singleton(type):
    """
    单例元类

    用法:
    >>> class A(metaclass=Singleton):
    >>>     pass
    """

    single_lock = RLock()

    def __call__(cls, *args, **kwargs):  # 创建cls的对象时候调用
        with Singleton.single_lock:
            if not hasattr(cls, "_instance"):
                cls._instance = super(Singleton, cls).__call__(*args, **kwargs)  # 创建cls的对象

        return cls._instance



from opentelemetry import trace
from opentelemetry.exporter.jaeger.thrift import JaegerExporter
from opentelemetry.sdk.resources import SERVICE_NAME, Resource
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import BatchSpanProcessor


def register_to_jaeger(name: str, host: str):
    """
    注册服务到jaeger，这样就可以发送tracer相关信息到jaeger服务器
    Args:
        name:  注册的服务明
        host:   jaeger地址

    Returns: TracerProvider

    """

    host = host.split(":")
    addr = host[0]
    if len(host) > 1:
        port = int(host[1])
    else:
        port = 6831

    provider = TracerProvider(resource=Resource.create({SERVICE_NAME: name}))
    trace.set_tracer_provider(provider)

    # create a JaegerExporter
    jaeger_exporter = JaegerExporter(
        agent_host_name=addr,
        agent_port=port,
    )

    # Create a BatchSpanProcessor and add the exporter to it
    span_processor = BatchSpanProcessor(jaeger_exporter)

    # add to the tracer
    trace.get_tracer_provider().add_span_processor(span_processor)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import collections.abc
import google.protobuf.empty_pb2
import grpc
import grpc.aio
import paperpilot_common.protobuf.test.test_pb2
import typing

_T = typing.TypeVar("_T")

class _MaybeAsyncIterator(collections.abc.AsyncIterator[_T], collections.abc.Iterator[_T], metaclass=abc.ABCMeta): ...

class _ServicerContext(grpc.ServicerContext, grpc.aio.ServicerContext):  # type: ignore
    ...

class TestPublicServiceStub:
    """测试公开接口"""

    def __init__(self, channel: typing.Union[grpc.Channel, grpc.aio.Channel]) -> None: ...
    Test: grpc.UnaryUnaryMultiCallable[
        google.protobuf.empty_pb2.Empty,
        paperpilot_common.protobuf.test.test_pb2.TestResult,
    ]
    """测试接口"""

class TestPublicServiceAsyncStub:
    """测试公开接口"""

    Test: grpc.aio.UnaryUnaryMultiCallable[
        google.protobuf.empty_pb2.Empty,
        paperpilot_common.protobuf.test.test_pb2.TestResult,
    ]
    """测试接口"""

class TestPublicServiceServicer(metaclass=abc.ABCMeta):
    """测试公开接口"""

    @abc.abstractmethod
    def Test(
        self,
        request: google.protobuf.empty_pb2.Empty,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.test.test_pb2.TestResult,
        collections.abc.Awaitable[paperpilot_common.protobuf.test.test_pb2.TestResult],
    ]:
        """测试接口"""

def add_TestPublicServiceServicer_to_server(
    servicer: TestPublicServiceServicer, server: typing.Union[grpc.Server, grpc.aio.Server]
) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.timestamp_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _Status:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _StatusEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_Status.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    RUNNING: _Status.ValueType  # 0
    """运行-无健康信息"""
    STOPPED: _Status.ValueType  # 1
    """停止"""
    STARTING: _Status.ValueType  # 2
    """启动中"""
    HEALTHY: _Status.ValueType  # 3
    """运行-健康"""
    UNHEALTHY: _Status.ValueType  # 4
    """运行-不健康"""
    UNKNOWN: _Status.ValueType  # 5
    """未知"""

class Status(_Status, metaclass=_StatusEnumTypeWrapper):
    """容器状态"""

RUNNING: Status.ValueType  # 0
"""运行-无健康信息"""
STOPPED: Status.ValueType  # 1
"""停止"""
STARTING: Status.ValueType  # 2
"""启动中"""
HEALTHY: Status.ValueType  # 3
"""运行-健康"""
UNHEALTHY: Status.ValueType  # 4
"""运行-不健康"""
UNKNOWN: Status.ValueType  # 5
"""未知"""
global___Status = Status

@typing_extensions.final
class ClientStatus(google.protobuf.message.Message):
    """采集端状态"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HOST_FIELD_NUMBER: builtins.int
    PROJECTS_FIELD_NUMBER: builtins.int
    TIME_FIELD_NUMBER: builtins.int
    host: builtins.str
    """采集端主机名"""
    @property
    def projects(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ClientProjectStatus]:
        """采集端项目状态"""
    @property
    def time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """采集时间"""
    def __init__(
        self,
        *,
        host: builtins.str = ...,
        projects: collections.abc.Iterable[global___ClientProjectStatus] | None = ...,
        time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["time", b"time"]) -> builtins.bool: ...
    def ClearField(
        self, field_name: typing_extensions.Literal["host", b"host", "projects", b"projects", "time", b"time"]
    ) -> None: ...

global___ClientStatus = ClientStatus

@typing_extensions.final
class ClientProjectStatus(google.protobuf.message.Message):
    """采集端项目状态"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_NAME_FIELD_NUMBER: builtins.int
    CONTAINERS_FIELD_NUMBER: builtins.int
    project_name: builtins.str
    """项目名"""
    @property
    def containers(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ClientContainerStatus]:
        """采集端容器状态"""
    def __init__(
        self,
        *,
        project_name: builtins.str = ...,
        containers: collections.abc.Iterable[global___ClientContainerStatus] | None = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing_extensions.Literal["containers", b"containers", "project_name", b"project_name"]
    ) -> None: ...

global___ClientProjectStatus = ClientProjectStatus

@typing_extensions.final
class ClientContainerStatus(google.protobuf.message.Message):
    """采集端容器状态"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    HOST_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    id: builtins.str
    """容器ID"""
    name: builtins.str
    """容器名"""
    host: builtins.str
    """容器主机名"""
    status: global___Status.ValueType
    """容器状态"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        name: builtins.str = ...,
        host: builtins.str = ...,
        status: global___Status.ValueType = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing_extensions.Literal["host", b"host", "id", b"id", "name", b"name", "status", b"status"]
    ) -> None: ...

global___ClientContainerStatus = ClientContainerStatus



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: paperpilot_common/protobuf/translation/translation.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n8paperpilot_common/protobuf/translation/translation.proto\x12\x0btranslation"\x89\x01\n\x12TranslationRequest\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\t\x12\x1c\n\x0fsource_language\x18\x02 \x01(\tH\x00\x88\x01\x01\x12\x1c\n\x0ftarget_language\x18\x03 \x01(\tH\x01\x88\x01\x01\x42\x12\n\x10_source_languageB\x12\n\x10_target_language"%\n\x13TranslationResponse\x12\x0e\n\x06result\x18\x01 \x01(\t2j\n\x18TranslationPublicService\x12N\n\ttranslate\x12\x1f.translation.TranslationRequest\x1a .translation.TranslationResponseb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "paperpilot_common.protobuf.translation.translation_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_TRANSLATIONREQUEST"]._serialized_start = 74
    _globals["_TRANSLATIONREQUEST"]._serialized_end = 211
    _globals["_TRANSLATIONRESPONSE"]._serialized_start = 213
    _globals["_TRANSLATIONRESPONSE"]._serialized_end = 250
    _globals["_TRANSLATIONPUBLICSERVICE"]._serialized_start = 252
    _globals["_TRANSLATIONPUBLICSERVICE"]._serialized_end = 358
# @@protoc_insertion_point(module_scope)



import grpc.aio

from paperpilot_common.utils.log import get_logger

from .wrapper import StubTraceWrapper


class GrpcClient:
    server_name: str
    server_host: str
    stub_cls: type
    _stub: object = None
    channel: grpc.aio.Channel | None

    def __init__(self):
        assert self.server_name, "server_name must be set"
        assert self.stub_cls, "stub_cls must be set"
        if not hasattr(self, "server_host"):
            from ..config import clients

            self.server_host = clients[self.server_name]["server_host"]

        self.channel = None
        self._stub = None
        self.logger = get_logger(f"grpc.client.{self.server_name}")

    def _connect(self):
        self.channel = grpc.aio.insecure_channel(self.server_host)
        self.logger.info(f"connect to {self.server_name} server {self.server_host}")
        self._stub = StubTraceWrapper(self.stub_cls(self.channel))

    @property
    def stub(self):
        if self._stub is None:
            self._connect()
        return self._stub

    def connect(self):
        self._connect()

    async def close(self):
        if self.channel:
            await self.channel.close(None)
            self.logger.debug(f"close {self.server_name}")

    def __repr__(self):
        return f"<GrpcClient {self.server_name}>"

    def __str__(self):
        return self.server_name



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: paperpilot_common/protobuf/monitor/client.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n/paperpilot_common/protobuf/monitor/client.proto\x12\x07monitor\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1bgoogle/protobuf/empty.proto"v\n\x0c\x43lientStatus\x12\x0c\n\x04host\x18\x01 \x01(\t\x12.\n\x08projects\x18\x02 \x03(\x0b\x32\x1c.monitor.ClientProjectStatus\x12(\n\x04time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp"_\n\x13\x43lientProjectStatus\x12\x14\n\x0cproject_name\x18\x01 \x01(\t\x12\x32\n\ncontainers\x18\x02 \x03(\x0b\x32\x1e.monitor.ClientContainerStatus"`\n\x15\x43lientContainerStatus\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0c\n\x04host\x18\x03 \x01(\t\x12\x1f\n\x06status\x18\x04 \x01(\x0e\x32\x0f.monitor.Status*Y\n\x06Status\x12\x0b\n\x07RUNNING\x10\x00\x12\x0b\n\x07STOPPED\x10\x01\x12\x0c\n\x08STARTING\x10\x02\x12\x0b\n\x07HEALTHY\x10\x03\x12\r\n\tUNHEALTHY\x10\x04\x12\x0b\n\x07UNKNOWN\x10\x05\x32R\n\x14MonitorClientService\x12:\n\tGetStatus\x12\x16.google.protobuf.Empty\x1a\x15.monitor.ClientStatusb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "paperpilot_common.protobuf.monitor.client_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_STATUS"]._serialized_start = 437
    _globals["_STATUS"]._serialized_end = 526
    _globals["_CLIENTSTATUS"]._serialized_start = 122
    _globals["_CLIENTSTATUS"]._serialized_end = 240
    _globals["_CLIENTPROJECTSTATUS"]._serialized_start = 242
    _globals["_CLIENTPROJECTSTATUS"]._serialized_end = 337
    _globals["_CLIENTCONTAINERSTATUS"]._serialized_start = 339
    _globals["_CLIENTCONTAINERSTATUS"]._serialized_end = 435
    _globals["_MONITORCLIENTSERVICE"]._serialized_start = 528
    _globals["_MONITORCLIENTSERVICE"]._serialized_end = 610
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import google.protobuf.timestamp_pb2
import paperpilot_common.protobuf.common.util_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class PaperId(google.protobuf.message.Message):
    """论文 ID"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    id: builtins.str
    def __init__(
        self,
        *,
        id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["id", b"id"]) -> None: ...

global___PaperId = PaperId

@typing_extensions.final
class PaperDetail(google.protobuf.message.Message):
    """论文详细信息"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    PROJECT_ID_FIELD_NUMBER: builtins.int
    TITLE_FIELD_NUMBER: builtins.int
    ABSTRACT_FIELD_NUMBER: builtins.int
    KEYWORDS_FIELD_NUMBER: builtins.int
    AUTHORS_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    PUBLICATION_YEAR_FIELD_NUMBER: builtins.int
    PUBLICATION_FIELD_NUMBER: builtins.int
    VOLUME_FIELD_NUMBER: builtins.int
    ISSUE_FIELD_NUMBER: builtins.int
    PAGES_FIELD_NUMBER: builtins.int
    URL_FIELD_NUMBER: builtins.int
    DOI_FIELD_NUMBER: builtins.int
    FILE_FIELD_NUMBER: builtins.int
    CREATE_TIME_FIELD_NUMBER: builtins.int
    UPDATE_TIME_FIELD_NUMBER: builtins.int
    EVENT_FIELD_NUMBER: builtins.int
    id: builtins.str
    """论文 ID"""
    project_id: builtins.str
    """所属项目 ID"""
    title: builtins.str
    """标题"""
    abstract: builtins.str
    """摘要"""
    @property
    def keywords(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """关键字列表"""
    @property
    def authors(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """作者列表"""
    @property
    def tags(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """tag列表"""
    publication_year: builtins.int
    """出版年份"""
    publication: builtins.str
    """出版方"""
    volume: builtins.str
    """卷号"""
    issue: builtins.str
    """期号"""
    pages: builtins.str
    """页码"""
    url: builtins.str
    doi: builtins.str
    file: builtins.str
    """文件 url"""
    @property
    def create_time(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def update_time(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    event: builtins.str
    """活动"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        project_id: builtins.str = ...,
        title: builtins.str = ...,
        abstract: builtins.str = ...,
        keywords: collections.abc.Iterable[builtins.str] | None = ...,
        authors: collections.abc.Iterable[builtins.str] | None = ...,
        tags: collections.abc.Iterable[builtins.str] | None = ...,
        publication_year: builtins.int = ...,
        publication: builtins.str = ...,
        volume: builtins.str = ...,
        issue: builtins.str = ...,
        pages: builtins.str = ...,
        url: builtins.str = ...,
        doi: builtins.str = ...,
        file: builtins.str = ...,
        create_time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        update_time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        event: builtins.str = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing_extensions.Literal["create_time", b"create_time", "update_time", b"update_time"]
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "abstract",
            b"abstract",
            "authors",
            b"authors",
            "create_time",
            b"create_time",
            "doi",
            b"doi",
            "event",
            b"event",
            "file",
            b"file",
            "id",
            b"id",
            "issue",
            b"issue",
            "keywords",
            b"keywords",
            "pages",
            b"pages",
            "project_id",
            b"project_id",
            "publication",
            b"publication",
            "publication_year",
            b"publication_year",
            "tags",
            b"tags",
            "title",
            b"title",
            "update_time",
            b"update_time",
            "url",
            b"url",
            "volume",
            b"volume",
        ],
    ) -> None: ...

global___PaperDetail = PaperDetail

@typing_extensions.final
class ListPaperRequest(google.protobuf.message.Message):
    """获取论文列表请求"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_ID_FIELD_NUMBER: builtins.int
    PAGE_FIELD_NUMBER: builtins.int
    PAGE_SIZE_FIELD_NUMBER: builtins.int
    ORDER_BY_FIELD_NUMBER: builtins.int
    project_id: builtins.str
    """项目 ID"""
    page: builtins.int
    """页码"""
    page_size: builtins.int
    """每页数量"""
    order_by: builtins.str
    """排序"""
    def __init__(
        self,
        *,
        project_id: builtins.str = ...,
        page: builtins.int = ...,
        page_size: builtins.int = ...,
        order_by: builtins.str = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "order_by", b"order_by", "page", b"page", "page_size", b"page_size", "project_id", b"project_id"
        ],
    ) -> None: ...

global___ListPaperRequest = ListPaperRequest

@typing_extensions.final
class ListPaperResponse(google.protobuf.message.Message):
    """获取论文列表响应"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PAPERS_FIELD_NUMBER: builtins.int
    TOTAL_FIELD_NUMBER: builtins.int
    NEXT_PAGE_FIELD_NUMBER: builtins.int
    @property
    def papers(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___PaperDetail]:
        """论文列表"""
    total: builtins.int
    """总数"""
    next_page: builtins.int
    """下一页"""
    def __init__(
        self,
        *,
        papers: collections.abc.Iterable[global___PaperDetail] | None = ...,
        total: builtins.int = ...,
        next_page: builtins.int = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing_extensions.Literal["next_page", b"next_page", "papers", b"papers", "total", b"total"]
    ) -> None: ...

global___ListPaperResponse = ListPaperResponse

@typing_extensions.final
class CreatePaperRequest(google.protobuf.message.Message):
    """添加论文请求"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_ID_FIELD_NUMBER: builtins.int
    PAPER_FIELD_NUMBER: builtins.int
    project_id: builtins.str
    """项目 ID"""
    @property
    def paper(self) -> global___PaperDetail: ...
    def __init__(
        self,
        *,
        project_id: builtins.str = ...,
        paper: global___PaperDetail | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["paper", b"paper"]) -> builtins.bool: ...
    def ClearField(
        self, field_name: typing_extensions.Literal["paper", b"paper", "project_id", b"project_id"]
    ) -> None: ...

global___CreatePaperRequest = CreatePaperRequest

@typing_extensions.final
class UpdateAttachmentRequest(google.protobuf.message.Message):
    """更新论文附件请求"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PAPER_ID_FIELD_NUMBER: builtins.int
    FILE_FIELD_NUMBER: builtins.int
    FETCH_METADATA_FIELD_NUMBER: builtins.int
    paper_id: builtins.str
    """论文 ID"""
    file: builtins.str
    """文件 url"""
    fetch_metadata: builtins.bool
    """是否自动获取元数据"""
    def __init__(
        self,
        *,
        paper_id: builtins.str = ...,
        file: builtins.str = ...,
        fetch_metadata: builtins.bool = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "fetch_metadata", b"fetch_metadata", "file", b"file", "paper_id", b"paper_id"
        ],
    ) -> None: ...

global___UpdateAttachmentRequest = UpdateAttachmentRequest

@typing_extensions.final
class UploadAttachmentRequest(google.protobuf.message.Message):
    """上传附件请求"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PAPER_ID_FIELD_NUMBER: builtins.int
    FETCH_METADATA_FIELD_NUMBER: builtins.int
    paper_id: builtins.str
    """论文 ID"""
    fetch_metadata: builtins.bool
    """是否自动获取元数据"""
    def __init__(
        self,
        *,
        paper_id: builtins.str = ...,
        fetch_metadata: builtins.bool = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing_extensions.Literal["fetch_metadata", b"fetch_metadata", "paper_id", b"paper_id"]
    ) -> None: ...

global___UploadAttachmentRequest = UploadAttachmentRequest

@typing_extensions.final
class UploadAttachmentResponse(google.protobuf.message.Message):
    """上传附件响应"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TOKEN_FIELD_NUMBER: builtins.int
    @property
    def token(self) -> paperpilot_common.protobuf.common.util_pb2.OssToken: ...
    def __init__(
        self,
        *,
        token: paperpilot_common.protobuf.common.util_pb2.OssToken | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["token", b"token"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["token", b"token"]) -> None: ...

global___UploadAttachmentResponse = UploadAttachmentResponse

@typing_extensions.final
class CreatePaperByLinkRequest(google.protobuf.message.Message):
    """通过链接创建论文请求"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_ID_FIELD_NUMBER: builtins.int
    LINK_FIELD_NUMBER: builtins.int
    project_id: builtins.str
    link: builtins.str
    def __init__(
        self,
        *,
        project_id: builtins.str = ...,
        link: builtins.str = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing_extensions.Literal["link", b"link", "project_id", b"project_id"]
    ) -> None: ...

global___CreatePaperByLinkRequest = CreatePaperByLinkRequest



from django.db import close_old_connections, reset_queries
from django.dispatch import Signal

# Triggered before each RPC
# Args provided by signal: request, context
grpc_request_started = Signal()

# Triggered after each RPC
# Args provided by signal: request, context
grpc_request_finished = Signal()

# Triggered if there was an error during RPC execution
# Args provided by signal: request, context, exception
grpc_got_request_exception = Signal()

# Reset database connections between requests
grpc_request_started.connect(reset_queries)
grpc_request_started.connect(close_old_connections)
grpc_request_finished.connect(close_old_connections)



from contextvars import ContextVar, Token
from typing import Any, Dict, Optional

_request_context: ContextVar[Dict[str, Any]] = ContextVar("request_context", default={})


class Context(object):
    package: str
    service: str
    method: str
    method_name: str
    metadata_dict: dict
    inited: bool
    req_id: str
    grpc_type: str

    def __getattr__(self, key: str) -> Any:
        value: Any = _request_context.get().get(key)
        return value

    def __setattr__(self, key: str, value: Any) -> None:
        _request_context.get()[key] = value


class WithContext(Context):
    def __init__(self) -> None:
        self._token: Optional[Token] = None

    def __enter__(self) -> "WithContext":
        if self._token:
            self._token = _request_context.set({})
        return self

    def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
        if self._token:
            _request_context.reset(self._token)


context_proxy: Context = Context()



from google.protobuf import any_pb2
from google.rpc import status_pb2

from paperpilot_common.exceptions import ApiException
from paperpilot_common.helper.field import datetime_to_timestamp
from paperpilot_common.middleware.server.auth import get_user
from paperpilot_common.protobuf.common import exce_pb2
from paperpilot_common.response import ResponseType
from paperpilot_common.response.types import ResponseData
from paperpilot_common.utils.log import get_logger
from paperpilot_common.utils.singleton import Singleton

try:
    from paperpilot_common.exceptions.configs import zq_exception_settings

    SENTRY_ENABLE = zq_exception_settings.SENTRY_ENABLE
except ImportError:
    SENTRY_ENABLE = True

if SENTRY_ENABLE:  # pragma: no cover
    try:
        import sentry_sdk
    except ImportError:
        SENTRY_ENABLE = False


def map_exception_to_response_type(exception):
    from django.core.exceptions import ObjectDoesNotExist, PermissionDenied, ValidationError
    from django.db import DatabaseError, IntegrityError
    from django.http import Http404
    from django.urls.exceptions import NoReverseMatch

    if isinstance(exception, ValidationError):
        # Django 验证错误，可以映射为参数错误的 ResponseType
        return ResponseType.ParamValidationFailed
    elif isinstance(exception, IntegrityError):
        # 数据库完整性错误，可以映射为数据库服务出错的 ResponseType
        return ResponseType.ParamValidationFailed
    elif isinstance(exception, DatabaseError):
        # 数据库错误，可以映射为数据库服务出错的 ResponseType
        return ResponseType.DatabaseError
    elif isinstance(exception, Http404):
        # HTTP 404 错误，可以映射为资源不存在的 ResponseType
        return ResponseType.ResourceNotFound
    elif isinstance(exception, ObjectDoesNotExist):
        # Django 对象不存在异常，可以映射为资源不存在的 ResponseType
        return ResponseType.ResourceNotFound
    elif isinstance(exception, NoReverseMatch):
        # Django URL 反向匹配异常，可以映射为请求接口不存在的 ResponseType
        return ResponseType.APINotFound
    elif isinstance(exception, PermissionDenied):
        # Django 权限拒绝异常，可以映射为用户无权限的 ResponseType
        return ResponseType.PermissionDenied
    elif isinstance(exception, ValueError):
        # 值错误异常，可以映射为参数错误的 ResponseType
        return ResponseType.ParamError
    elif isinstance(exception, KeyError):
        # 键错误异常，可以映射为参数错误的 ResponseType
        return ResponseType.ParamError
    elif isinstance(exception, FileNotFoundError):
        # 文件未找到异常，可以映射为资源不存在的 ResponseType
        return ResponseType.ResourceNotFound
    elif isinstance(exception, IOError):
        # 输入/输出异常，可以映射为服务器错误的 ResponseType
        return ResponseType.ServerError
    elif isinstance(exception, TypeError):
        # 类型错误异常，可以映射为参数错误的 ResponseType
        return ResponseType.ParamError
    elif isinstance(exception, AttributeError):
        # 属性错误异常，可以映射为参数错误的 ResponseType
        return ResponseType.ParamError
    elif isinstance(exception, NotImplementedError):
        # 未实现异常，可以映射为接口未实现的 ResponseType
        return ResponseType.Unimplemented
    elif isinstance(exception, Exception):
        # 其他异常，可以映射为服务器错误的 ResponseType
        return ResponseType.ServerError


class ApiExceptionHandler(metaclass=Singleton):
    logger = get_logger("server.interceptor.exception")

    def _convert(self, exc: Exception) -> ApiException:
        if not isinstance(exc, ApiException):
            self.logger.debug(f"convert exception {exc.__class__.__name__} to ApiException")
            response_type = map_exception_to_response_type(exc)
            exc = ApiException(response_type, inner=exc, detail=str(exc), record=True)

        return exc

    def notify_sentry(self, data: ResponseData, exc: ApiException) -> str:
        """
        通知sentry
        :param exc: Api异常
        :param data: 响应数据
        :return: sentry_id
        """
        user = get_user()
        if user.is_authenticated:
            sentry_sdk.api.set_tag("role", "user")
            sentry_sdk.api.set_user(
                {
                    "id": user.id,
                }
            )
        else:
            sentry_sdk.api.set_tag("role", "guest")

        # 默认异常汇报
        sentry_sdk.api.set_tag("exception_type", exc.response_type.name)
        sentry_sdk.set_context(
            "exp_info",
            {
                "eid": data["data"]["eid"],
                "code": data["code"],
                "detail": data["detail"],
                "msg": data["msg"],
            },
        )

        sentry_sdk.set_context(
            "details",
            data,
        )

        eid = sentry_sdk.api.capture_exception(exc.inner if exc.inner else exc)  # 优先发送内部错误
        self.logger.debug(f"notify sentry {eid}")

        return eid

    def grpc_handle(self, exc: Exception) -> status_pb2.Status:
        exc = self._convert(exc)
        if hasattr(exc, "inner"):
            self.logger.exception(exc.inner)
        response_data = exc.response_data

        if exc.record:  # 如果需要记录
            if SENTRY_ENABLE:
                sentry_id = self.notify_sentry(response_data, exc)
                response_data["data"]["sentry_id"] = sentry_id

        protos = self.get_grpc_protos(response_data)

        detail = any_pb2.Any()
        detail.Pack(protos)

        return status_pb2.Status(
            code=exc.response_type.grpc_status_code.value[0],
            message=protos.detail,  # 面向开发者
            details=[detail],
        )

    def get_grpc_protos(self, response_data) -> exce_pb2.ApiException:
        """
        获取grpc protos
        :param response_data: 响应数据
        :return: grpc protos
        """
        if response_data["data"]["info"]:
            info = exce_pb2.ExceptionInfo(
                type=response_data["data"]["info"]["type"],
                value=response_data["data"]["info"]["value"],
                traceback=response_data["data"]["info"]["traceback"],
                inner_type=response_data["data"]["info"]["inner_type"],
                inner_value=response_data["data"]["info"]["inner_value"],
            )
        else:
            info = None

        return exce_pb2.ApiException(
            code=response_data["code"],
            detail=response_data["detail"],
            message=response_data["msg"],
            data=exce_pb2.ApiExceptionData(
                eid=response_data["data"]["eid"],
                sentry_id=response_data["data"]["sentry_id"],
                time=datetime_to_timestamp(response_data["data"]["time"]),
                info=info,
            ),
        )



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import collections.abc
import google.protobuf.empty_pb2
import grpc
import grpc.aio
import paperpilot_common.protobuf.monitor.server_pb2
import typing

_T = typing.TypeVar("_T")

class _MaybeAsyncIterator(collections.abc.AsyncIterator[_T], collections.abc.Iterator[_T], metaclass=abc.ABCMeta): ...

class _ServicerContext(grpc.ServicerContext, grpc.aio.ServicerContext):  # type: ignore
    ...

class MonitorPublicServiceStub:
    """监控公开服务"""

    def __init__(self, channel: typing.Union[grpc.Channel, grpc.aio.Channel]) -> None: ...
    GetStatus: grpc.UnaryUnaryMultiCallable[
        google.protobuf.empty_pb2.Empty,
        paperpilot_common.protobuf.monitor.server_pb2.ServerStatus,
    ]
    """获取后端状态"""

class MonitorPublicServiceAsyncStub:
    """监控公开服务"""

    GetStatus: grpc.aio.UnaryUnaryMultiCallable[
        google.protobuf.empty_pb2.Empty,
        paperpilot_common.protobuf.monitor.server_pb2.ServerStatus,
    ]
    """获取后端状态"""

class MonitorPublicServiceServicer(metaclass=abc.ABCMeta):
    """监控公开服务"""

    @abc.abstractmethod
    def GetStatus(
        self,
        request: google.protobuf.empty_pb2.Empty,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.monitor.server_pb2.ServerStatus,
        collections.abc.Awaitable[paperpilot_common.protobuf.monitor.server_pb2.ServerStatus],
    ]:
        """获取后端状态"""

def add_MonitorPublicServiceServicer_to_server(
    servicer: MonitorPublicServiceServicer, server: typing.Union[grpc.Server, grpc.aio.Server]
) -> None: ...



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2

from paperpilot_common.protobuf.test import test_pb2 as paperpilot__common_dot_protobuf_dot_test_dot_test__pb2


class TestPublicServiceStub(object):
    """测试公开接口"""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Test = channel.unary_unary(
            "/test.TestPublicService/Test",
            request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_test_dot_test__pb2.TestResult.FromString,
        )


class TestPublicServiceServicer(object):
    """测试公开接口"""

    def Test(self, request, context):
        """测试接口"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_TestPublicServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "Test": grpc.unary_unary_rpc_method_handler(
            servicer.Test,
            request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_test_dot_test__pb2.TestResult.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler("test.TestPublicService", rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class TestPublicService(object):
    """测试公开接口"""

    @staticmethod
    def Test(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/test.TestPublicService/Test",
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            paperpilot__common_dot_protobuf_dot_test_dot_test__pb2.TestResult.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: paperpilot_common/protobuf/ai/ai.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n&paperpilot_common/protobuf/ai/ai.proto\x12\x02\x61i"\\\n\tGptResult\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\t\x12,\n\rfinish_reason\x18\x02 \x01(\x0e\x32\x10.ai.FinishReasonH\x00\x88\x01\x01\x42\x10\n\x0e_finish_reason"*\n\nGptRequest\x12\x0c\n\x04text\x18\x01 \x01(\t\x12\x0e\n\x06\x61\x63tion\x18\x02 \x01(\t*U\n\x0c\x46inishReason\x12\x08\n\x04NULL\x10\x00\x12\x08\n\x04STOP\x10\x01\x12\n\n\x06LENGTH\x10\x02\x12\x11\n\rFUNCTION_CALL\x10\x03\x12\x12\n\x0e\x43ONTENT_FILTER\x10\x04\x32\x34\n\nGptService\x12&\n\x03\x41sk\x12\x0e.ai.GptRequest\x1a\r.ai.GptResult0\x01\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "paperpilot_common.protobuf.ai.ai_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_FINISHREASON"]._serialized_start = 184
    _globals["_FINISHREASON"]._serialized_end = 269
    _globals["_GPTRESULT"]._serialized_start = 46
    _globals["_GPTRESULT"]._serialized_end = 138
    _globals["_GPTREQUEST"]._serialized_start = 140
    _globals["_GPTREQUEST"]._serialized_end = 182
    _globals["_GPTSERVICE"]._serialized_start = 271
    _globals["_GPTSERVICE"]._serialized_end = 323
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _FinishReason:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _FinishReasonEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_FinishReason.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    NULL: _FinishReason.ValueType  # 0
    """未结束"""
    STOP: _FinishReason.ValueType  # 1
    """生成结束"""
    LENGTH: _FinishReason.ValueType  # 2
    """生成长度达到最大值"""
    FUNCTION_CALL: _FinishReason.ValueType  # 3
    """需要调用函数"""
    CONTENT_FILTER: _FinishReason.ValueType  # 4
    """生成内容被过滤"""

class FinishReason(_FinishReason, metaclass=_FinishReasonEnumTypeWrapper):
    """OpenAI FinishReason"""

NULL: FinishReason.ValueType  # 0
"""未结束"""
STOP: FinishReason.ValueType  # 1
"""生成结束"""
LENGTH: FinishReason.ValueType  # 2
"""生成长度达到最大值"""
FUNCTION_CALL: FinishReason.ValueType  # 3
"""需要调用函数"""
CONTENT_FILTER: FinishReason.ValueType  # 4
"""生成内容被过滤"""
global___FinishReason = FinishReason

@typing_extensions.final
class GptResult(google.protobuf.message.Message):
    """GPT 响应"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CONTENT_FIELD_NUMBER: builtins.int
    FINISH_REASON_FIELD_NUMBER: builtins.int
    content: builtins.str
    """生成内容"""
    finish_reason: global___FinishReason.ValueType
    """结束原因"""
    def __init__(
        self,
        *,
        content: builtins.str = ...,
        finish_reason: global___FinishReason.ValueType | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing_extensions.Literal["_finish_reason", b"_finish_reason", "finish_reason", b"finish_reason"],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "_finish_reason", b"_finish_reason", "content", b"content", "finish_reason", b"finish_reason"
        ],
    ) -> None: ...
    def WhichOneof(
        self, oneof_group: typing_extensions.Literal["_finish_reason", b"_finish_reason"]
    ) -> typing_extensions.Literal["finish_reason"] | None: ...

global___GptResult = GptResult

@typing_extensions.final
class GptRequest(google.protobuf.message.Message):
    """GPT 请求"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TEXT_FIELD_NUMBER: builtins.int
    ACTION_FIELD_NUMBER: builtins.int
    text: builtins.str
    """所选文本"""
    action: builtins.str
    """操作"""
    def __init__(
        self,
        *,
        text: builtins.str = ...,
        action: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["action", b"action", "text", b"text"]) -> None: ...

global___GptRequest = GptRequest



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: paperpilot_common/protobuf/common/util.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n,paperpilot_common/protobuf/common/util.proto\x12\x04util"0\n\x04\x44\x61te\x12\x0c\n\x04year\x18\x01 \x01(\x05\x12\r\n\x05month\x18\x02 \x01(\x05\x12\x0b\n\x03\x64\x61y\x18\x03 \x01(\x05"i\n\x08OssToken\x12\x15\n\raccess_key_id\x18\x01 \x01(\t\x12\x0e\n\x06policy\x18\x02 \x01(\t\x12\x11\n\tsignature\x18\x03 \x01(\t\x12\x0c\n\x04host\x18\x04 \x01(\t\x12\x15\n\rcallback_body\x18\x05 \x01(\tb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "paperpilot_common.protobuf.common.util_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_DATE"]._serialized_start = 54
    _globals["_DATE"]._serialized_end = 102
    _globals["_OSSTOKEN"]._serialized_start = 104
    _globals["_OSSTOKEN"]._serialized_end = 209
# @@protoc_insertion_point(module_scope)



import uuid
from contextvars import ContextVar
from typing import Any, Callable

import grpc

from paperpilot_common.middleware.server.base import AsyncServerMiddleware
from paperpilot_common.utils.log import get_logger


class UserContext:
    """
    用户上下文
    """

    id: uuid.UUID | None = None

    def __init__(self, user_id: str | None = None):
        if user_id:
            self.id = uuid.UUID(user_id)
        else:
            self.id = None

    @property
    def is_anonymous(self) -> bool:
        return self.id is None

    @property
    def is_authenticated(self) -> bool:
        return self.id is not None

    def __repr__(self) -> str:
        return f"<UserContext user_id={self.id}>"


user_context: ContextVar[UserContext | None] = ContextVar("user_context", default=None)


def get_user() -> UserContext:
    user = user_context.get()
    if user is None:
        user_context.set(UserContext())

    return user_context.get()


class AuthMixin:
    @property
    def user(self) -> UserContext:
        return get_user()


class AuthMiddleware(AsyncServerMiddleware):
    logger = get_logger("server.interceptor.auth")

    def __init__(self):
        try:
            from django.conf import settings

            self.auth_metadata_key = getattr(settings, "AUTH_METADATA_KEY", "x-kong-jwt-claim-user_id")
        except ImportError:
            self.auth_metadata_key = "x-kong-jwt-claim-user_id"

    async def intercept(
        self,
        method: Callable,
        request_or_iterator: Any,
        context: grpc.ServicerContext,
        method_name: str,
    ) -> Any:
        try:
            metadata = dict(context.invocation_metadata())
            user_id = metadata.get(self.auth_metadata_key, None)
            user_context.set(UserContext(user_id))

            return await self.call(method, request_or_iterator, context)
        except Exception as e:
            self.logger.exception(e)
            raise e



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import google.protobuf.timestamp_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class ApiException(google.protobuf.message.Message):
    """标准异常响应"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CODE_FIELD_NUMBER: builtins.int
    DETAIL_FIELD_NUMBER: builtins.int
    MESSAGE_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    code: builtins.str
    """错误码，详见：paperpilot-common-python/paperpilot_common.response.ResponseType"""
    detail: builtins.str
    """获取异常详情(面向开发者)"""
    message: builtins.str
    """获取异常信息(面向用户)"""
    @property
    def data(self) -> global___ApiExceptionData:
        """其他数据"""
    def __init__(
        self,
        *,
        code: builtins.str = ...,
        detail: builtins.str = ...,
        message: builtins.str = ...,
        data: global___ApiExceptionData | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["data", b"data"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "code", b"code", "data", b"data", "detail", b"detail", "message", b"message"
        ],
    ) -> None: ...

global___ApiException = ApiException

@typing_extensions.final
class ApiExceptionData(google.protobuf.message.Message):
    """详细异常数据"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EID_FIELD_NUMBER: builtins.int
    SENTRY_ID_FIELD_NUMBER: builtins.int
    TIME_FIELD_NUMBER: builtins.int
    INFO_FIELD_NUMBER: builtins.int
    eid: builtins.str
    """异常 id(可选，仅限异常被数据库记录时返回)"""
    sentry_id: builtins.str
    """sentry id(可选，仅限异常被sentry记录时返回)"""
    @property
    def time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """时间"""
    @property
    def info(self) -> global___ExceptionInfo:
        """python异常信息(可选，仅限debug时返回)"""
    def __init__(
        self,
        *,
        eid: builtins.str = ...,
        sentry_id: builtins.str = ...,
        time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        info: global___ExceptionInfo | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["info", b"info", "time", b"time"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "eid", b"eid", "info", b"info", "sentry_id", b"sentry_id", "time", b"time"
        ],
    ) -> None: ...

global___ApiExceptionData = ApiExceptionData

@typing_extensions.final
class ExceptionInfo(google.protobuf.message.Message):
    """python异常信息"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TYPE_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    TRACEBACK_FIELD_NUMBER: builtins.int
    INNER_TYPE_FIELD_NUMBER: builtins.int
    INNER_VALUE_FIELD_NUMBER: builtins.int
    type: builtins.str
    """异常类型"""
    value: builtins.str
    """异常内容"""
    @property
    def traceback(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """异常堆栈"""
    inner_type: builtins.str
    """内部异常类型"""
    inner_value: builtins.str
    """内部异常内容"""
    def __init__(
        self,
        *,
        type: builtins.str = ...,
        value: builtins.str = ...,
        traceback: collections.abc.Iterable[builtins.str] | None = ...,
        inner_type: builtins.str = ...,
        inner_value: builtins.str = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "inner_type",
            b"inner_type",
            "inner_value",
            b"inner_value",
            "traceback",
            b"traceback",
            "type",
            b"type",
            "value",
            b"value",
        ],
    ) -> None: ...

global___ExceptionInfo = ExceptionInfo



from typing import Dict, List, TypedDict, Union

JSONVal = Union[None, bool, str, float, int, List["JSONVal"], Dict[str, "JSONVal"]]


class ResponseData(TypedDict, total=True):
    code: str
    detail: str
    msg: str
    data: JSONVal


# class ApiExceptionResponse(Response):
#     api_request_data: Dict[str, JSONVal]
#     exception_data: ApiException
#     exception: bool



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import collections.abc
import google.protobuf.empty_pb2
import grpc
import grpc.aio
import paperpilot_common.protobuf.user.auth_pb2
import paperpilot_common.protobuf.user.user_pb2
import typing

_T = typing.TypeVar("_T")

class _MaybeAsyncIterator(collections.abc.AsyncIterator[_T], collections.abc.Iterator[_T], metaclass=abc.ABCMeta): ...

class _ServicerContext(grpc.ServicerContext, grpc.aio.ServicerContext):  # type: ignore
    ...

class AuthPublicServiceStub:
    """认证公开接口（可匿名访问）"""

    def __init__(self, channel: typing.Union[grpc.Channel, grpc.aio.Channel]) -> None: ...
    Login: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.user.auth_pb2.LoginRequest,
        paperpilot_common.protobuf.user.auth_pb2.LoginResponse,
    ]
    """登录"""
    Refresh: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.user.auth_pb2.RefreshTokenRequest,
        paperpilot_common.protobuf.user.auth_pb2.RefreshTokenResponse,
    ]
    """刷新token"""
    Logout: grpc.UnaryUnaryMultiCallable[
        google.protobuf.empty_pb2.Empty,
        google.protobuf.empty_pb2.Empty,
    ]
    """登出"""
    Register: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.user.user_pb2.CreateUserRequest,
        paperpilot_common.protobuf.user.auth_pb2.LoginResponse,
    ]
    """注册"""
    SendSmsCode: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.user.auth_pb2.SendSmsCodeRequest,
        google.protobuf.empty_pb2.Empty,
    ]
    """发送短信验证码"""
    CountPhone: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.user.auth_pb2.CountPhoneRequest,
        paperpilot_common.protobuf.user.auth_pb2.CountResponse,
    ]
    """获取手机号数量"""
    CountUsername: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.user.auth_pb2.CountUsernameRequest,
        paperpilot_common.protobuf.user.auth_pb2.CountResponse,
    ]
    """获取用户名数量"""

class AuthPublicServiceAsyncStub:
    """认证公开接口（可匿名访问）"""

    Login: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.user.auth_pb2.LoginRequest,
        paperpilot_common.protobuf.user.auth_pb2.LoginResponse,
    ]
    """登录"""
    Refresh: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.user.auth_pb2.RefreshTokenRequest,
        paperpilot_common.protobuf.user.auth_pb2.RefreshTokenResponse,
    ]
    """刷新token"""
    Logout: grpc.aio.UnaryUnaryMultiCallable[
        google.protobuf.empty_pb2.Empty,
        google.protobuf.empty_pb2.Empty,
    ]
    """登出"""
    Register: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.user.user_pb2.CreateUserRequest,
        paperpilot_common.protobuf.user.auth_pb2.LoginResponse,
    ]
    """注册"""
    SendSmsCode: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.user.auth_pb2.SendSmsCodeRequest,
        google.protobuf.empty_pb2.Empty,
    ]
    """发送短信验证码"""
    CountPhone: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.user.auth_pb2.CountPhoneRequest,
        paperpilot_common.protobuf.user.auth_pb2.CountResponse,
    ]
    """获取手机号数量"""
    CountUsername: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.user.auth_pb2.CountUsernameRequest,
        paperpilot_common.protobuf.user.auth_pb2.CountResponse,
    ]
    """获取用户名数量"""

class AuthPublicServiceServicer(metaclass=abc.ABCMeta):
    """认证公开接口（可匿名访问）"""

    @abc.abstractmethod
    def Login(
        self,
        request: paperpilot_common.protobuf.user.auth_pb2.LoginRequest,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.user.auth_pb2.LoginResponse,
        collections.abc.Awaitable[paperpilot_common.protobuf.user.auth_pb2.LoginResponse],
    ]:
        """登录"""
    @abc.abstractmethod
    def Refresh(
        self,
        request: paperpilot_common.protobuf.user.auth_pb2.RefreshTokenRequest,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.user.auth_pb2.RefreshTokenResponse,
        collections.abc.Awaitable[paperpilot_common.protobuf.user.auth_pb2.RefreshTokenResponse],
    ]:
        """刷新token"""
    @abc.abstractmethod
    def Logout(
        self,
        request: google.protobuf.empty_pb2.Empty,
        context: _ServicerContext,
    ) -> typing.Union[google.protobuf.empty_pb2.Empty, collections.abc.Awaitable[google.protobuf.empty_pb2.Empty]]:
        """登出"""
    @abc.abstractmethod
    def Register(
        self,
        request: paperpilot_common.protobuf.user.user_pb2.CreateUserRequest,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.user.auth_pb2.LoginResponse,
        collections.abc.Awaitable[paperpilot_common.protobuf.user.auth_pb2.LoginResponse],
    ]:
        """注册"""
    @abc.abstractmethod
    def SendSmsCode(
        self,
        request: paperpilot_common.protobuf.user.auth_pb2.SendSmsCodeRequest,
        context: _ServicerContext,
    ) -> typing.Union[google.protobuf.empty_pb2.Empty, collections.abc.Awaitable[google.protobuf.empty_pb2.Empty]]:
        """发送短信验证码"""
    @abc.abstractmethod
    def CountPhone(
        self,
        request: paperpilot_common.protobuf.user.auth_pb2.CountPhoneRequest,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.user.auth_pb2.CountResponse,
        collections.abc.Awaitable[paperpilot_common.protobuf.user.auth_pb2.CountResponse],
    ]:
        """获取手机号数量"""
    @abc.abstractmethod
    def CountUsername(
        self,
        request: paperpilot_common.protobuf.user.auth_pb2.CountUsernameRequest,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.user.auth_pb2.CountResponse,
        collections.abc.Awaitable[paperpilot_common.protobuf.user.auth_pb2.CountResponse],
    ]:
        """获取用户名数量"""

def add_AuthPublicServiceServicer_to_server(
    servicer: AuthPublicServiceServicer, server: typing.Union[grpc.Server, grpc.aio.Server]
) -> None: ...



from starlette.endpoints import HTTPEndpoint
from starlette.responses import JSONResponse
from starlette.routing import Route

from paperpilot_common.utils.log import get_logger


class HealthView(HTTPEndpoint):
    logger = get_logger("health.view")

    async def get(self, request):
        return JSONResponse({})


routes = [
    Route("/", HealthView),
]
